import logging
from unittest.case import TestCase

from django_request_logger import middleware, logger


class FakeUser(object):
    def __init__(self, username):
        self.username = username


class FakeRequest(object):
    def __init__(self, username, absolute_url, raw_post_data, meta):
        self.user = FakeUser(username)
        self.absolute_url = absolute_url
        self.raw_post_data = raw_post_data
        self.META = meta

    def build_absolute_uri(self):
        return self.absolute_url


class AddDjangoRequestFilterTest(TestCase):
    def test_no_request(self):
        filter = logger.AddDjangoRequestFilter()
        record = logging.LogRecord("name", 0, None, None, "msg", None, None)

        self.assertEqual(1, filter.filter(record))
        self.assertEqual("", record.username)
        self.assertEqual("", record.client_ip)
        self.assertEqual("", record.absolute_url)
        self.assertEqual("", record.raw_post_data)
        self.assertFalse(hasattr(record, 'data'))

    def test_with_request(self):
        filter = logger.AddDjangoRequestFilter()
        record = logging.LogRecord("name", 0, None, None, "msg", None, None)

        request = FakeRequest("testuser", "http://fake.url/test", "post=data", {'REMOTE_ADDR': "123.4.5.67"})
        middleware._set_request(request)

        self.assertEqual(1, filter.filter(record))
        self.assertEqual("testuser", record.username)
        self.assertEqual("123.4.5.67", record.client_ip)
        self.assertEqual("http://fake.url/test", record.absolute_url)
        self.assertEqual("post=data", record.raw_post_data)
        self.assertEqual("testuser", record.data['username'])
        self.assertEqual("123.4.5.67", record.data['client_ip'])
        self.assertEqual("http://fake.url/test", record.data['absolute_url'])
        self.assertEqual("post=data", record.data['raw_post_data'])

        request = FakeRequest("", "http://fake.url/test", None,
                {'REMOTE_ADDR': "1.2.3.4", 'HTTP_X_FORWARDED_FOR': "5.6.7.8"})
        request.user = None
        middleware._set_request(request)

        self.assertEqual(1, filter.filter(record))
        self.assertEqual("", record.username)
        self.assertEqual("1.2.3.4 [5.6.7.8]", record.client_ip)
        self.assertEqual("http://fake.url/test", record.absolute_url)
        self.assertEqual(None, record.raw_post_data)



import threading


_thread_local = threading.local()


def get_request():
    return getattr(_thread_local, 'current_django_request', None)


def _set_request(value):
    _thread_local.current_django_request = value


class StoreRequestMiddleware(object):
    def process_request(self, request):
        _set_request(request)

    def process_response(self, request, response):
        _set_request(None)
        return response

    # Note: do not clear the request in process_exception() - we want the request details in the error message logged
    # by "django.request" and process_response() gets called later anyway.



import logging

import middleware


def add_filter_to_all_handlers():
    filter = AddDjangoRequestFilter()
    for handler in logging._handlers.values():
        handler.addFilter(filter)


def get_client_ip(request):
    if request is None:
        return ""

    client_ip = request.META.get('REMOTE_ADDR')
    if 'HTTP_X_FORWARDED_FOR' in request.META:
        client_ip += " [%s]" % request.META['HTTP_X_FORWARDED_FOR']
    return client_ip


class AddDjangoRequestFilter(object):
    def filter(self, record):
        # Initialise custom fields to empty, so the formatter doesn't raise an error if there is no request or user
        record.client_ip = ""
        record.username = ""
        record.absolute_url = ""
        record.raw_post_data = ""

        request = middleware.get_request()
        if request is not None:
            record.client_ip = get_client_ip(request)
            record.absolute_url = request.build_absolute_uri()

            try:
                record.raw_post_data = request.body
            except Exception as e:
                 record.raw_post_data = '<ERROR: %s>' % e.message

            if request.user is not None:
                record.username = request.user.username or "_"

            # The "data" dictionary is used by the Sentry log handler.
            if not hasattr(record, 'data'):
                record.data = {}
            record.data['client_ip'] = record.client_ip
            record.data['absolute_url'] = record.absolute_url
            record.data['raw_post_data'] = record.raw_post_data
            record.data['username'] = record.username

        return 1



from distutils.core import setup

setup(
    name='DjangoRequestLogger',
    version='0.2.0',
    author='Evgeny Morozov',
    author_email='python-code@realityexists.net',
    packages=['django_request_logger', 'django_request_logger.test'],
    url='http://pypi.python.org/pypi/DjangoRequestLogger/',
    license='LICENSE.txt',
    description='Django app to add current username, client IP, etc. to logging output.',
    long_description='''
This Django app adds extra fields to log records emitted
by the standard Python logging. Currently these include:

* username - Username of the currently logged in Django user, if any. If no
  user is logged in the value is "_" (distinct from empty string, which means
  that the information is not available).
* client_ip - IP address of the client browser, obtained from REMOTE_ADDR and
  HTTP_X_FORWARDED_FOR.
* absolute_url - the absolute URL of the current request, including the query
  string.
* raw_post_data - the raw POST data for the current request, if any.

These fields are then available to be used in your logging formatter (see
example below). They are also visible in Django Sentry as additional data (if
you have Django Sentry installed).
'''
)
